<?php
namespace Teknisa\CMP\Controllers;

use \Setforms2\Shared\ORM\DateTime;

class ContaPagar {

    protected $contaPagarService;
    
    public function __construct($contaPagarService) {
        $this->contaPagarService = $contaPagarService;
    }
    
    
    public function excluiNFContaPagar($cdfildest, $cdfillanc, $nrautoforn, $orcamentoFinanceiro, $empresa = '') {
        
        try {
        
            $this->contaPagarService->proceExcluiContaPagarPrevista($cdfildest, $cdfillanc, $nrautoforn);
            
            if($orcamentoFinanceiro) {
                
                $idgercontpgt = $this->contaPagarService->retornaGeraContaPagarFornecedor($cdfildest, $cdfillanc, $nrautoforn);
                
                if($idgercontpgt) {
                    $cdclassfina = self::verificaClassificacaoFinanceira($cdfildest, $cdfillanc, $nrautoforn);
                    if($cdclassfina) {
                        $autoForne = $this->contaPagarService->retornaAutoforneContaPagar($cdfildest, $cdfillanc, $nrautoforn);
                        
                        $paramsProceCriaVencimento = array(
                            "CDFILDEST"     =>  $cdfildest,
                            "CDFILLANC"     =>  $cdfillanc,
                            "NRAUTOFORN"    =>  $nrautoforn
                        );
                        
                        $paramsProceOrcamentoOk = array(
                            'CDEMPRESA'     => $empresa ? $empresa : $autoForne[0]['CDEMPRESA'],
                            'CDCLASSFINA'   => $cdclassfina
                        );
                        
                        foreach($autoForne as $af) {
                            $paramsProceCriaVencimento['VRTOTAL_REST'] = $af['VRTOTAL_REST'];
                            $paramsProceCriaVencimento['DTPROGENTAF']  = substr($af["DTPROGENTAF"], 0, 10);
                            
                            $this->contaPagarService->proceCriaVencimento($paramsProceCriaVencimento);
                            /**
                             * 26/04/2019 - Foi definido que, para este processo, os parâmetros tem a seguinte equivalência:
                             * ---------------------------------
                             * | Tabela          |    AF       |
                             * ---------------------------------
                             * | CDFILIALVENC    =  CDFILDEST  |
                             * | NRLANCTONFVENC  =  NRAUTOFORN |
                             * ---------------------------------
                             */
                            $trabVencimento = $this->contaPagarService->retornaTrabVencimento($cdfildest, $nrautoforn);
                            
                            foreach($trabVencimento as $trabVenc) {
                                $paramsProceOrcamentoOk['CDFILIAL']      = $trabVenc['CDFILIALVENC'];
                                $paramsProceOrcamentoOk['DTLANCAMENTO']  = $trabVenc['DTVENCIMENTO'];
                                $paramsProceOrcamentoOk['VRVALOR']       = $trabVenc['VRVENCIMENTO'];
                                
                                $retornoProceOrcamento = $this->contaPagarService->proceOrcamentoOk($paramsProceOrcamentoOk);
                                
                                $orcamentoNaoEstaOk = !self::isOrcamentoOk($retornoProceOrcamento);
                                
                                if ($orcamentoNaoEstaOk) {
                                    $paramsProceOrcamentoOk['VRVALOR'] = -$trabVenc['VRVENCIMENTO'];
                                    
                                    $this->contaPagarService->proceOrcamentoOk($paramsProceOrcamentoOk);
                                    break;
                                }
                            }
                            $this->contaPagarService->deletaTrabVencimento($cdfildest, $nrautoforn);
                            if ($orcamentoNaoEstaOk) {
                                break;
                            }
                        }
                        
                        if ($orcamentoNaoEstaOk) {
                            foreach($autoForne as $af) {
                                $paramsProceCriaVencimento['VRTOTAL_REST'] = $af['VRTOTAL_REST'];
                                $paramsProceCriaVencimento['DTPROGENTAF'] = substr($af["DTPROGENTAF"], 0, 10);
                                
                                $this->contaPagarService->proceCriaVencimento($paramsProceCriaVencimento);
                                
                                $trabVencimento = $this->contaPagarService->retornaTrabVencimento();
                                
                                foreach($trabVencimento as $trabVenc) {
                                    $paramsProceOrcamentoOk['CDFILIAL']      = $trabVenc['CDFILIALVENC'];
                                    $paramsProceOrcamentoOk['DTLANCAMENTO']  = $trabVenc['DTVENCIMENTO'];
                                    $paramsProceOrcamentoOk['VRVALOR']       = -$trabVenc['VRVENCIMENTO'];
                                    
                                    $this->contaPagarService->proceOrcamentoOk($paramsProceOrcamentoOk);
                                }
                            }
                        }
                        
                    }
                    
                }
                
            }
            
            return true;
        
        } catch(\Exception $e) {
            
            throw new \Exception($e->getMessage(), 10);
            
        }
        
    }
    
    
    public function verificaClassificacaoFinanceira($cdfildest, $cdfillanc, $nrautoforn) {
        
        $classificacaoFinanceira = $this->contaPagarService->retornaClassificacaoFinanceira($cdfildest, $cdfillanc, $nrautoforn);
        $cdclassfina = $classificacaoFinanceira ? $classificacaoFinanceira['CDCLASSFINA'] : false;
        
        if(!$cdclassfina) {
            
            $cdtipooper = $this->contaPagarService->retornaTipoOper($cdfildest);
            
            $classificacaoTipoOperador = $this->contaPagarService->retornaClassificacaoTipoOperador($cdtipooper);
            $cdclassfina = $classificacaoTipoOperador ? $classificacaoTipoOperador['CDCLASSFINA'] : false;
            
        }
        
        return $cdclassfina;
        
    }
    
    
    public function geraNFContaPagar($cdfildest, $cdfillanc, $nrautoforn) {
        
        try {
            
            $this->contaPagarService->proceGeraContaPagarPrevista($cdfildest, $cdfillanc, $nrautoforn);
            
        } catch(\Exception $e) {
            
            throw new \Exception($e->getMessage(), 20);
            
        }
        
    }
    
    /**
     * A procedure avalia essa condição e a retorna em forma de números.
     * 0 - Erro.
     * 1 - Ok.
     */
    public function isOrcamentoOk($retornoProceOrcamento) {
        return $retornoProceOrcamento['P_RESULTADO'] == 1;
    }
}
   