<?php
namespace Teknisa\CMP\Services;

class DataProcessamentoService
{
    protected $entityManager;
    protected $connection;
    protected $environment;

    public function __construct($entityManager, $environment)
    {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
    }
    
    /**
     * Verifica se a filial passada por parâmetro é a central de compras
     * 
     * @param {string} $cdfilial Código da Filial
     * 
     * @return {boolean} Retorna True caso o filial passada por parametro for 
     * uma central de compras, ao contrario, retorna False
     * 
    */
    public function verificaFilialCentral($cdfilial)
    {
        $cdfilial = empty($cdfilial) ? $this->environment->getCdFilial() : $cdfilial;
        $central = $this->getCentralCompras($cdfilial);
        return $central === $cdfilial;
    }
    
    private function getCentralCompras($cdfilial)
    {
        $sql = "SELECT NVL(CDFILCENTCMP, CDFILIAL) AS CDFILCENTCMP
                  FROM FILIAL
                 WHERE CDFILIAL = :P_CDFILIAL
                   AND NRORG = :NRORG";
        $params = [
            "P_CDFILIAL" => $cdfilial,
            "NRORG" => $this->environment->getNrOrgTrab()
        ];
        
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return !empty($result) ? $result[0]['CDFILCENTCMP'] : '';
        } catch (\Exception $e){
            throw $e;
        }
    }
    
    public function getDataProcessamento($cdfilial)
    {
        $sql = "SELECT CASE NVL(PARAMGERAL.IDUTILDTPROC, 'N') WHEN 'S'
                        THEN NVL(PARAMFILIAL.DTPROCESSA, SYSDATE)
                        ELSE SYSDATE
                       END AS DTPROCESSA
                  FROM PARAMFILIAL JOIN PARAMGERAL ON PARAMFILIAL.NRORG = PARAMGERAL.NRORG
                 WHERE PARAMFILIAL.CDFILIAL = :P_CDFILIAL
                   AND PARAMFILIAL.NRORG = :NRORG";
        $params = [
            "P_CDFILIAL" => $cdfilial,
            "NRORG" => $this->environment->getNrOrgTrab()
        ];
        
        try {
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return substr($result[0]['DTPROCESSA'], 0, 10);
        } catch (\Exception $e){
            throw $e;
        }
    }
}
